#include "scripts.hpp"

#include <iostream>
#include <lua.hpp>

#include "globals.hpp"
#include "world.hpp"

#include "BasicDemo.h"

#include <set>


#include "BulletDynamics/Dynamics/btRigidBody.h"
#include "BulletDynamics/Dynamics/btDiscreteDynamicsWorld.h"
#include "btBulletDynamicsCommon.h"

#include "triangulate.hpp"

#define DO_STACK_DUMP

// optional stack dump mischief
#ifdef DO_STACK_DUMP
   void stack_dump(lua_State *stack); // forward declare
   #define STACK_DUMP stack_dump(L); (void)0
#else
   #define STACK_DUMP (void)0
#endif

using namespace std;
lua_State *L;


void l_pushTableNum(int i, char* key , double value) {

lua_getglobal(L,"coins");
    lua_pushnumber(L, i);

    lua_gettable(L,-2);
        lua_pushnumber(L, value);

        lua_setfield(L, -2, key);
        //lua_pop(L,-1);
        lua_pushnumber(L, i);

        lua_insert(L, -2);

        lua_settable(L,-3);

    lua_setglobal(L, "coins");

    //lua_pop(L,1);


}

int exportParamsToLua(int i)
{
lua_settop(L, 0);

 //lua_pushnumber(ls, mass);
    //std::cout << "Elements in stack:" << lua_gettop(L) << std::endl;

    double mass = 1.0/(World::getInstance()->coins[i]->getInvMass());
    l_pushTableNum(i,"mass",mass);

    btVector3 localInvInertia= World::getInstance()->coins[i]->getInvInertiaDiagLocal();
    double Ixx = 1.0/localInvInertia.getX();
    l_pushTableNum(i,"Ixx",Ixx);

    double Iyy = 1.0/localInvInertia.getY();
    l_pushTableNum(i,"Iyy",Iyy);

    double Izz = 1.0/localInvInertia.getZ();
    l_pushTableNum(i,"Izz",Izz);

    double linDamp =  World::getInstance()->coins[i]->getLinearDamping();
    l_pushTableNum(i,"linDamp",linDamp);

    double angDamp =  World::getInstance()->coins[i]->getAngularDamping();
    l_pushTableNum(i,"angDamp",angDamp);

    double friction =  World::getInstance()->coins[i]->getFriction();
    l_pushTableNum(i,"friction",friction);

    double posX=  World::getInstance()->coins[i]->getCenterOfMassPosition().getX();
    l_pushTableNum(i,"posX",posX);

    double posY=  World::getInstance()->coins[i]->getCenterOfMassPosition().getY();
    l_pushTableNum(i,"posY",posY);

    double posZ=  World::getInstance()->coins[i]->getCenterOfMassPosition().getZ();
    l_pushTableNum(i,"posZ",posZ);


//  lua_pop(L, 1);

  //std::cout << "Elements in stack:" << lua_gettop(L) << std::endl;

 return 1;
}

void luaInitialize()
{

  std::cout << "** Test Lua embedding" << endl;
  std::cout << "** Init Lua" << endl;

  L = luaL_newstate();

  std::cout << "** Load the (optional) standard libraries, to have the print function" << endl;
  luaL_openlibs(L);
  lua_register(L, "hitTest", lua_hitTest);
  lua_register(L, "createZone", lua_createZone);
  lua_register(L, "createCoin", lua_createCoin);
  lua_register(L, "queryZone", lua_queryZone);
  lua_register(L, "syncZones", lua_syncZones);
  //lua_register(L, "triangulate", lua_triangulate);
  std::cout << "** Load chunk. without executing it" << endl;
  if (luaL_dofile(L, "lua/effects.lua")) {
    std::cerr << "Something went wrong loading the chunk (syntax error?)" << endl;
    std::cerr << lua_tostring(L, -1) << endl;
    lua_pop(L,1);

    }

  //coinGetField(1,"mass");

};

double coinGetField(int coin, char* field)
{
   lua_settop(L, 0);
   lua_getglobal(L, "coins");

    //if (lua_pcall(L,0,1,0)!= 0)
    //    std::cerr << "error running function" << lua_tostring(L, -1);

        if (!lua_istable(L, -1))
            std::cerr <<"coins is not a table";
        lua_pushinteger(L,coin);
        lua_gettable(L,-2);
        //if (!lua_isnumber(L, -1))
        //    std::cerr << "invalid component in params";
            if (!lua_istable(L, -1))
                std::cerr <<"coin" << coin << "is not a table";

	        lua_pushstring(L,field);
	        lua_gettable(L,-2);
	            //std::cerr << (double)lua_tonumber(L, -1) << std::endl;
	            double res = (double)lua_tonumber(L, -1);
	        lua_pop(L,1);
	    lua_pop(L,1);
    lua_pop(L,1);



	//std::cout << "Elements in stack:" << lua_gettop(L) << std::endl;

	return res;
};


double lua_getField(char* tableName, int index, char* field)
{
	lua_settop(L, 0);
	lua_getglobal(L, tableName);

	if (!lua_istable(L, -1))
		std::cerr <<"coins is not a table";
		lua_pushinteger(L,index);
			lua_gettable(L,-2);

			if (!lua_istable(L, -1))
				std::cerr <<"element " << index << " is not a table";
			lua_pushstring(L,field);
				lua_gettable(L,-2);
				double res = (double)lua_tonumber(L, -1);
			lua_pop(L,1);
	    lua_pop(L,1);
    lua_pop(L,1);
	return res;
};


std::pair<short,short> lua_getScore ()
{
	lua_settop(L, 0);
	lua_getglobal(L, "score");

	if (!lua_istable(L, -1))
		std::cerr <<"score is not a table";
		lua_pushinteger(L,0);
			lua_gettable(L,-2);
			short score0 = (short)lua_tonumber(L, -1);
			lua_pop(L,1);

		lua_pushinteger(L,1);
			lua_gettable(L,-2);
			short score1 = (short)lua_tonumber(L, -1);
			lua_pop(L,1);

	std::pair<short,short> res(score0,score1);

	return res;
};



int coinGetNum()
{
   lua_getglobal(L, "coins");
   lua_len(L,-1);
   int res = lua_tonumber(L, -1);
   lua_pop(L,1);
   lua_pop(L,1);
   return res;
};


double getLuaParam(char* name)
{
   lua_getglobal(L, name);
       std::cerr << name <<":" << (double)lua_tonumber(L, -1) << std::endl;
       double res = (double)lua_tonumber(L, -1);
   lua_pop(L,1);
   return res;
};


void luaStep()
{
    //lua_settop(L, 0);
	lua_getglobal(L, "applyEffects");
	    lua_pcall(L,0,0,NULL);
	lua_pop(L,1);
};

void importParamsFromLua(int i)
{

		btTransform transform;
		World::getInstance()->coins[i]->getMotionState()->getWorldTransform(transform);
		//startTransform.setIdentity();
		transform.setOrigin(SCALING*btVector3(
											btScalar(coinGetField(i,"posX")),
											btScalar(coinGetField(i,"posY")),
											btScalar(coinGetField(i,"posZ"))));

		//World::getInstance()->coins[i]->setWorldTransform(startTransform);
		World::getInstance()->coins[i]->getMotionState()->setWorldTransform(transform);

		btVector3 localInertia(0,0,0);
		localInertia=btVector3(1.0/coinGetField(i,"Ixx"),1.0/coinGetField(i,"Iyy"),1.0/coinGetField(i,"Izz"));
		World::getInstance()->coins[i]->setInvInertiaDiagLocal(localInertia);


		//btRigidBody::btRigidBodyConstructionInfo rbInfo(coinGetField(i,"mass"),myMotionState,colShape,localInertia);
		//coin* body = (coin*)new btRigidBody(rbInfo);

		//body->setInvInertiaDiagLocal(btVector3(0.001f,0.001f,1.f));
		World::getInstance()->coins[i]->setFriction(coinGetField(i,"friction"));
		//World::getInstance()->coins[i]->setRestitution(0.0f);
		//World::getInstance()->coins[i]->setDamping(0.3f,0.3f);
		//World::getInstance()->coins[i]->setCcdMotionThreshold(0.001f);
		//World::getInstance()->coins[i]->setCcdSweptSphereRadius(4.f);
}



void stack_dump(lua_State *stack)
{
   int stackSize = lua_gettop(stack);

   std::cout << "Stack Dump: " << std::endl;

   for (int i = stackSize; i > 0; --i)
   {
      int absoluteIndex = i;
      int relativeIndex = stackSize - i + 1;

      std::cout << "Index[" << absoluteIndex << " / -" << relativeIndex << "] = ";

      int type = lua_type(stack, i);
      switch (type)
      {
      case LUA_TSTRING:
         std::cout << lua_tostring(stack, i);
         break;
      case LUA_TBOOLEAN:
         std::cout << lua_toboolean(stack, i);
         break;
      case LUA_TNUMBER:
         std::cout << lua_tonumber(stack, i);
         break;
      case LUA_TUSERDATA:
      case LUA_TLIGHTUSERDATA:
         std::cout << lua_touserdata(stack, i);
         break;
      default:
         std::cout << lua_typename(stack, type);
      }

      std::cout << std::endl;
   }

   std::cout << std::endl;
}

std::set<int> luaGetCoinKeys()
{
    lua_settop(L, 0);
	std::set<int> keys;
	lua_getglobal(L, "getCoinKeys");
	    lua_pcall(L,0,1,NULL);
		lua_pushnil(L);
			while (lua_next(L, -2))
			{
			    lua_pushvalue(L,-2);
				int key = lua_tonumber(L, -2);
				keys.insert(key);
				lua_pop(L,2);
			}
	lua_settop(L,0);
	return keys;

		//std::set<int> keys = lua
};


std::set<int> lua_getKeys(char* targetTable)
{
    lua_settop(L, 0);
	std::set<int> keys;
	lua_getglobal(L, "getKeys");
	lua_pushstring(L,targetTable);

      if (lua_pcall(L,1,1,NULL) != 0)
        {
			//error(L, "error running function `f': %s", lua_tostring(L, -1));
	  }
		lua_pushnil(L);
		//lua_pop(L,-1);
		STACK_DUMP;
			while (lua_next(L, -2))
			{
			    lua_pushvalue(L,-2);
				int key = lua_tonumber(L, -2);
				keys.insert(key);
				//STACK_DUMP;
				lua_pop(L,2);
			}
	lua_settop(L,0);
	return keys;

		//std::set<int> keys = lua
};



int lua_hitTest(lua_State *L)
{
	bool hitFloor;

    double v1x = luaL_checknumber(L,1);
    double v1y = luaL_checknumber(L,2);
    double v1z = luaL_checknumber(L,3);

    double v2x = luaL_checknumber(L,4);
    double v2y = luaL_checknumber(L,5);
    double v2z = luaL_checknumber(L,6);
    lua_settop(L, 0);

    btVector3 rayFrom = btVector3 (v1x,v1y,v1z);

    btVector3 rayTo = btVector3 (v2x,v2y,v2z);

	//std::cout << v1x << ";" << v1y << ";" << v1z << ";" << v2x << ";" << v2y << ";" << v2z <<"\n";

    btCollisionWorld::AllHitsRayResultCallback rayCallback(rayFrom,rayTo);

    World::getInstance()->getDynamicsWorld()->rayTest(rayFrom,rayTo,rayCallback);

	hitFloor=false;
	if (rayCallback.hasHit())
	{
		for (int i=0; i<rayCallback.m_collisionObjects.size();i++)
		{
			if (rayCallback.m_collisionObjects[i]==World::getInstance()->ground)
			{
				hitFloor=true;
			}
		}
	};

	/*
	if (rayCallback.hasHit())
	{
     if (rayCallback.m_collisionObjects.size()>2)
		 hitFloor=false;
	 else
		 hitFloor=true;
	}
	else  hitFloor=false;
	*/

	lua_pushboolean(L, hitFloor);
	return 1;
}

int lua_createZone(lua_State *L)
{


    lua_pushstring(L,"radius");
    lua_gettable(L,-2);
    double r = lua_tonumber(L,-1);
    lua_pop(L,1);

    lua_pushstring(L,"anchor");
    lua_gettable(L,-2);
    int anchor = lua_tointeger(L,-1);
    lua_pop(L,1);

    lua_pushstring(L,"type");
    lua_gettable(L,-2);
    const char* type = lua_tostring(L,-1);
    lua_pop(L,1);

    lua_pushstring(L,"offset");
    lua_gettable(L,-2);
    lua_pushnil(L);


    std::vector<double> offset;

    while (lua_next(L, -2))
    {
        lua_pushvalue(L,-2);
        offset.push_back(lua_tonumber(L, -2));

        lua_pop(L,2);
    }
	lua_pop(L,1);

    btTransform startTransform;
    startTransform.setIdentity ();
    startTransform.setOrigin (btVector3(offset[0],offset[1],0.0f));



	//lua_getglobal(L, "levelMap");
    lua_pushstring(L,"contour");
	lua_gettable(L,-2);
	//STACK_DUMP;
	//lua_gettable(L,-2);

	lua_pushnil(L);


	std::vector<double> vertex;
	//std::vector<std::vector<double> > triangle;
	btTriangleMesh* zoneMesh = new btTriangleMesh(true,false);
	Vector2dVector contour;
	Vector2dVector result;
	//STACK_DUMP;

    while (lua_next(L, -2))
    {
        lua_pushvalue(L,-2);
		lua_pushnil(L);
		//STACK_DUMP;

	    while (lua_next(L, -3))
		{
			lua_pushvalue(L,-2);
			lua_pushnil(L);
			//STACK_DUMP;
			vertex.push_back(lua_tonumber(L, -3));
			//double v1 = lua_tonumber(L, -2);
			lua_pop(L,3);
		}
			//vertex.push_back(0.f);
			contour.push_back(btVector3(vertex[0],vertex[1],0.f));
			vertex.clear();
			//STACK_DUMP;
			lua_pop(L,2);
			//lua_pop(L,1);
			//STACK_DUMP;
    }
	lua_pop(L,2);

	Triangulate::Process(contour,result);

	Vector2dVector levelTriangles = result;
    int tcount = levelTriangles.size()/3;

    for (int i=0; i<tcount; i++)
    {
         btVector3 &p1 = levelTriangles[i*3+0];
         btVector3 &p2 = levelTriangles[i*3+1];
         btVector3 &p3 = levelTriangles[i*3+2];
		btVector3 v1 = p2 - p1;
		btVector3 v2 = p3 - p2;

		btVector3 n1=v1.cross(v2);
		if (n1.getZ()<=0.f)
		{
			btVector3& p4 =  p1;

			p1=p3;

			p3=p4;
		}



        //printf("Triangle %d => (%0.0f,%0.0f) (%0.0f,%0.0f) (%0.0f,%0.0f)\n",i+1,p1.getX(),p1.getY(),p2.getX(),p2.getY(),p3.getX(),p3.getY());
        zoneMesh->addTriangle(btVector3(p1.getX(),p1.getY(),1.f),btVector3(p2.getX(),p2.getY(),1.f),btVector3(p3.getX(),p3.getY(),1.f),true);
    }


/*
    lua_pushstring(L,"triangles");
	lua_gettable(L,-2);

	lua_pushnil(L);

	std::vector<double> vertex;
	std::vector<std::vector<double> > triangle;
	btTriangleMesh* groundMesh = new btTriangleMesh(true,false);

    while (lua_next(L, -2))
    {
        lua_pushvalue(L,-2);
		lua_pushnil(L);

	    while (lua_next(L, -3))
		    {
			lua_pushvalue(L,-2);
			lua_pushnil(L);

			while (lua_next(L, -3))
				{
					lua_pushvalue(L,-2);
					vertex.push_back(lua_tonumber(L, -2));
					lua_pop(L,2);
				}
			triangle.push_back(vertex);
			vertex.clear();
			lua_pop(L,2);
			}
		groundMesh->addTriangle(btVector3(triangle[0][0],triangle[0][1],1.0),btVector3(triangle[1][0],triangle[1][1],1.0),btVector3(triangle[2][0],triangle[2][1], 1.0),true);
		std::cout << std::endl << triangle[0][0] << "," << triangle[0][1] << "," << -10.0 << std::endl << triangle[1][0] << "," << triangle[1][1] << "," << -10.0 << std::endl << triangle[2][0] << "," << triangle[2][1] << "," << 5.0 ;
		triangle.clear();
		lua_pop(L,2);
    }
	lua_pop(L,1);

  	lua_pop(L,1);
	*/
	int id =  lua_tointeger(L,-1);


	btPairCachingGhostObject* ghostObject = new btPairCachingGhostObject();
    ghostObject->setWorldTransform(startTransform);
    World::getInstance()->m_broadphase->getOverlappingPairCache()->setInternalGhostPairCallback(new btGhostPairCallback());

	btBvhTriangleMeshShape* colShape = new btBvhTriangleMeshShape(zoneMesh,true);
    btConvexShape* colSphere = new btSphereShape(4.0f);
	World::getInstance()->m_collisionShapes.push_back(colShape);
	ghostObject->setCollisionShape (colShape);

    ghostObject->setCollisionFlags (btCollisionObject::CF_CHARACTER_OBJECT|btCollisionObject::CF_NO_CONTACT_RESPONSE);

	World::getInstance()->getDynamicsWorld()->addCollisionObject(ghostObject,btBroadphaseProxy::CharacterFilter, btBroadphaseProxy::StaticFilter|btBroadphaseProxy::DefaultFilter);

	World::getInstance()->zones[id]=ghostObject;


    //btGeneric6DofConstraint* dof6 = new btGeneric6DofConstraint(*ghostObject, tr,false);
	/*
	dof6->setLinearLowerLimit(btVector3(1,1,1));
    dof6->setLinearUpperLimit(btVector3(-1,-1,-1));
    dof6->setAngularLowerLimit(btVector3(0.f,0.f,1));
    dof6->setAngularUpperLimit(btVector3(1.e-3f,1.e-3f,-1));                                        //dof6->setAngularLowerLimit(btVector3(1,1,1));
	*/

    return 0;

}

int lua_createCoin(lua_State *L)
{

		btCollisionShape* colShape = new btCylinderShapeZ(btVector3(SCALING*4.f,SCALING*4.f,SCALING*0.7f));
		//colShape->setLocalScaling(btVector3(SCALING*1,SCALING*1,SCALING*0.2));
		//btCollisionShape* colShape = new btSphereShape(btScalar(1.));
		World::getInstance()->m_collisionShapes.push_back(colShape);

		/// Create Dynamic Objects
		btTransform startTransform;
		startTransform.setIdentity();
		startTransform.setOrigin(btVector3(0,0,0));

		btTransform groundTransform;
		groundTransform.setIdentity();
		groundTransform.setOrigin(btVector3(0,0,0));


	    lua_pushstring(L,"posX");
		lua_gettable(L,-2);
		double posX = lua_tonumber(L,-1);
		lua_pop(L,1);

	    lua_pushstring(L,"posY");
		lua_gettable(L,-2);
		double posY = lua_tonumber(L,-1);
		lua_pop(L,1);

	    lua_pushstring(L,"posZ");
		lua_gettable(L,-2);
		double posZ = lua_tonumber(L,-1);
		lua_pop(L,1);

		lua_pushstring(L,"Ixx");
		lua_gettable(L,-2);
		double Ixx = lua_tonumber(L,-1);
		lua_pop(L,1);

		lua_pushstring(L,"Iyy");
		lua_gettable(L,-2);
		double Iyy = lua_tonumber(L,-1);
		lua_pop(L,1);

		lua_pushstring(L,"Izz");
		lua_gettable(L,-2);
		double Izz = lua_tonumber(L,-1);
		lua_pop(L,1);

		lua_pushstring(L,"mass");
		lua_gettable(L,-2);
		double mass = lua_tonumber(L,-1);
		lua_pop(L,1);

		lua_pushstring(L,"friction");
		lua_gettable(L,-2);
		double friction = lua_tonumber(L,-1);
		lua_pop(L,1);

		lua_pushstring(L,"player");
		lua_gettable(L,-2);
		double player = lua_tonumber(L,-1);
		lua_pop(L,1);

		lua_pop(L,1);

		//lua_pop(L,-1);

		int id =  lua_tointeger(L,-1);

		lua_pop(L,-1);

		btVector3 localInertia(0,0,0);
		colShape->calculateLocalInertia(mass,localInertia);

        startTransform.setOrigin(SCALING*btVector3(btScalar(posX),btScalar(posY),btScalar(0.7f)));
        localInertia=btVector3(Ixx,Iyy,Izz);


		//using motionstate is recommended, it provides interpolation capabilities, and only synchronizes 'active' objects
		btDefaultMotionState* myMotionState = new btDefaultMotionState(startTransform);
		btRigidBody::btRigidBodyConstructionInfo rbInfo(mass,myMotionState,colShape,localInertia);
		coin* body = new coin(rbInfo);

		World::getInstance()->coins[id]=body;
					//body->setInvInertiaDiagLocal(btVector3(0.001f,0.001f,1.f));
		body->setFriction(friction);
					body->setRestitution(0.0f);
					body->setDamping(0.3f,0.3f);
					body->setCcdMotionThreshold(1e-7f);
					body->setCcdSweptSphereRadius(0.5f);
					body->player=player;
					World::getInstance()->getDynamicsWorld()->addRigidBody(body);


                    btTransform tr;
                    tr.setIdentity();
                    tr=groundTransform;
                    btGeneric6DofConstraint* dof6 = new btGeneric6DofConstraint(*body, tr,false);
                    dof6->setLinearLowerLimit(btVector3(1,1,0.0f));
                    dof6->setLinearUpperLimit(btVector3(-1,-1,0.71f));
                    //dof6->setAngularLowerLimit(btVector3(0.f,0.f,1));
                    //dof6->setAngularUpperLimit(btVector3(1.e-3f,1.e-3f,-1));                                        //dof6->setAngularLowerLimit(btVector3(1,1,1));
                    //dof6->setAngularUpperLimit(btVector3(-1,-1,-1));
                    //World::getInstance()->getDynamicsWorld()->addConstraint(dof6,true);

		//body->setLinearFactor(btVector3(1.0f,1.0f,0.1f));
		//body->setAngularFactor(btVector3(0,0,1.f));
		//body->setDamping(btScalar(100.f),btScalar(100.f));
		body->setRollingFriction(1.f);
	return 0;
}


int lua_queryZone(lua_State *L)
{
    //lua_pushstring(L,"posX");
	//lua_gettable(L,-2);


	int ignoreCoin = lua_tonumber(L,-1);
	lua_pop(L,1);
	int zoneId = lua_tonumber(L,-1);


	int numArgs = 0;
    btManifoldArray   manifoldArray;
    btBroadphasePairArray& pairArray = World::getInstance()->zones[zoneId]->getOverlappingPairCache()->getOverlappingPairArray();
    int numPairs = pairArray.size();
	std::set<int> result;
      for (int i=0;i<numPairs;i++)
      {
         manifoldArray.clear();

         const btBroadphasePair& pair = pairArray[i];

         //unless we manually perform collision detection on this pair, the contacts are in the dynamics world paircache:
         btBroadphasePair* collisionPair = World::getInstance()->getDynamicsWorld()->getPairCache()->findPair(pair.m_pProxy0,pair.m_pProxy1);
         if (!collisionPair)
            continue;

         if (collisionPair->m_algorithm)
            collisionPair->m_algorithm->getAllContactManifolds(manifoldArray);

         for (int j=0;j<manifoldArray.size();j++)
         {
            btPersistentManifold* manifold = manifoldArray[j];
			if (manifold->getBody0()!=World::getInstance()->coins[ignoreCoin])
			{
				btScalar directionSign = manifold->getBody0() == World::getInstance()->zones[zoneId] ? btScalar(-1.0) : btScalar(1.0);
				for (int p=0;p<manifold->getNumContacts();p++)
				{
             		const btManifoldPoint&pt = manifold->getContactPoint(p);
					if (pt.getDistance()<0.f)
					{
						const btVector3& ptA = pt.getPositionWorldOnA();
						const btVector3& ptB = pt.getPositionWorldOnB();
						const btVector3& normalOnB = pt.m_normalWorldOnB;

						std::map<short, coin*>::const_iterator it = MapSearchByValue<std::map<short, coin*>,coin*>(World::getInstance()->coins,(coin*)manifold->getBody0());
						if (it!=World::getInstance()->coins.end())
						{
							result.insert(it->first);
							//lua_pushinteger(L,it->first);
							//numArgs++;
							//return 1;
						}
					}
				}
			};
         }
      }

	//lua_pushinteger(L,0);
	//numArgs++;
	for (std::set<int>::iterator it1 = result.begin();it1!=result.end();it1++)
	{
		lua_pushinteger(L,*it1);
	}

    return result.size();
}


int lua_syncZones(lua_State *L)
{
	World::getInstance()->syncZones();
	return 0;
}


Vector2dVector lua_triangulate()
{

    lua_getglobal(L, "levelMap");
    //lua_pushstring(L,"triangles");
	//lua_gettable(L,-2);

	lua_pushnil(L);


	std::vector<double> vertex;
	//std::vector<std::vector<double> > triangle;
	//btTriangleMesh* groundMesh = new btTriangleMesh(true,false);
	Vector2dVector contour;
	Vector2dVector result;
	STACK_DUMP;

    while (lua_next(L, -2))
    {
        lua_pushvalue(L,-2);
		lua_pushnil(L);


	    while (lua_next(L, -3))
		{
			lua_pushvalue(L,-2);
			lua_pushnil(L);

			vertex.push_back(lua_tonumber(L, -3));
			//double v1 = lua_tonumber(L, -2);
			lua_pop(L,3);
		}
			//vertex.push_back(0.f);
			contour.push_back(btVector3(vertex[0],vertex[1],0.f));
			vertex.clear();

			lua_pop(L,2);
			//lua_pop(L,1);

    }

		Triangulate::Process(contour,result);

		//groundMesh->addTriangle(btVector3(triangle[0][0],triangle[0][1],0.0),btVector3(triangle[1][0],triangle[1][1],0.0),btVector3(triangle[2][0],triangle[2][1], 0.0),true);
		//std::cout << std::endl << triangle[0][0] << "," << triangle[0][1] << "," << -10.0 << std::endl << triangle[1][0] << "," << triangle[1][1] << "," << -10.0 << std::endl << triangle[2][0] << "," << triangle[2][1] << "," << 5.0 ;
		//triangle.clear();

    return result;

}


