


#include "BasicDemo.h"
#include "GlutStuff.h"
#include "BulletCollision/CollisionDispatch/btInternalEdgeUtility.h"
///btBulletDynamicsCommon.h is the main Bullet include file, contains most common include files.
#include "btBulletDynamicsCommon.h"

#include <stdio.h> //printf debugging
#include "GLDebugDrawer.h"

#include "scripts.hpp"
#include "coin.hpp"
#include "globals.hpp"

#include <set>
#include <algorithm>
#include <iterator>
#include <iostream>

static GLDebugDrawer gDebugDraw;


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>


#include "triangulate.hpp"

///User can override this material combiner by implementing gContactAddedCallback and setting body0->m_collisionFlags |= btCollisionObject::customMaterialCallback;
inline btScalar	calculateCombinedFriction(float friction0,float friction1)
{
	//return 0.f;
	btScalar friction = friction0 * friction1;

	const btScalar MAX_FRICTION  = 1.f;
	if (friction < -MAX_FRICTION)
		friction = -MAX_FRICTION;
	if (friction > MAX_FRICTION)
		friction = MAX_FRICTION;
	return friction;

}

inline btScalar	calculateCombinedRestitution(float restitution0,float restitution1)
{
	return restitution0 * restitution1;
}


///////////////////////////////////////////////////////////////


static bool CustomMaterialCombinerCallback(btManifoldPoint& cp,	const btCollisionObjectWrapper* colObj0Wrap,int partId0,int index0,const btCollisionObjectWrapper* colObj1Wrap,int partId1,int index1)
{

	btAdjustInternalEdgeContacts(cp,colObj1Wrap,colObj0Wrap, partId1,index1);
	//btAdjustInternalEdgeContacts(cp,colObj1,colObj0, partId1,index1, BT_TRIANGLE_CONVEX_BACKFACE_MODE);
	//btAdjustInternalEdgeContacts(cp,colObj1,colObj0, partId1,index1, BT_TRIANGLE_CONVEX_DOUBLE_SIDED+BT_TRIANGLE_CONCAVE_DOUBLE_SIDED);

	float friction0 = colObj0Wrap->getCollisionObject()->getFriction();
	float friction1 = colObj1Wrap->getCollisionObject()->getFriction();
	float restitution0 = colObj0Wrap->getCollisionObject()->getRestitution();
	float restitution1 = colObj1Wrap->getCollisionObject()->getRestitution();
	/*
	if (colObj0Wrap->getCollisionObject()->getCollisionFlags() & btCollisionObject::CF_CUSTOM_MATERIAL_CALLBACK)
	{
		friction0 = 1.0;//partId0,index0
		restitution0 = 0.f;
	}
	if (colObj1Wrap->getCollisionObject()->getCollisionFlags() & btCollisionObject::CF_CUSTOM_MATERIAL_CALLBACK)
	{
		if (index1&1)
		{
			friction1 = 1.0f;//partId1,index1
		} else
		{
			friction1 = 0.f;
		}
		restitution1 = 0.f;
	}
	*/
	cp.m_combinedFriction = calculateCombinedFriction(friction0,friction1);
	//cp.m_combinedFriction = 1;
	cp.m_combinedRestitution = calculateCombinedRestitution(restitution0,restitution1);

    cp.m_normalWorldOnB = btVector3(0.f,0.f,1.f);

	//this return value is currently ignored, but to be on the safe side: return false if you don't calculate friction
	return true;
}






void BasicDemo::clientMoveAndDisplay()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    static float time = 0;
	//simple dynamics world doesn't handle fixed-time-stepping
	float ms = getDeltaTimeMicroseconds();

    float dt = ms / 1000000.f;

	time += dt;

	///step the simulation
	if ((m_dynamicsWorld) && (Playback))
	{
		m_dynamicsWorld->stepSimulation(dt,30,1.0f/180.f);
		//optional but useful: debug drawing
		m_dynamicsWorld->debugDrawWorld();
	}



	if (!m_freeCam)
    {
        //m_cameraTargetPosition=pickedBody->getCenterOfMassPosition();
    }

    	for( std::map<short, coin*>::iterator it = coins.begin(); it != coins.end(); ++it)
        {
            exportParamsToLua(it->first);
        }
		//moving zones
		for( std::map<short, btPairCachingGhostObject*>::iterator it = zones.begin(); it != zones.end(); ++it)
        {
            int anchor = lua_getField("zones", it->first, "anchor");
			if (anchor != 0)
			{
				btTransform b1;
				coins[anchor]->getMotionState()->getWorldTransform(b1);
				b1.setOrigin(btVector3(b1.getOrigin().getX(),b1.getOrigin().getY(),0.0f));
				it->second->setWorldTransform(b1);
			}
        }


    if (time>0.01)
    {
        time=0;

        Playback = !isAtRest();

        luaStep();

		syncCoins();

    }
	renderme();

	glFlush();

	swapBuffers();

}


void BasicDemo::displayCallback(void) {

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	renderme();

	//optional but useful: debug drawing to detect problems
	if (m_dynamicsWorld)
		m_dynamicsWorld->debugDrawWorld();

	glFlush();
	swapBuffers();
}



int bottomTag = 1;

void	BasicDemo::initPhysics()
{
	setTexturing(false);
	setShadows(false);

	setCameraDistance(btScalar(SCALING*100.));

	///collision configuration contains default setup for memory, collision setup
	m_collisionConfiguration = new btDefaultCollisionConfiguration();
	//m_collisionConfiguration->setConvexConvexMultipointIterations();

	gContactAddedCallback = CustomMaterialCombinerCallback;

	///use the default collision dispatcher. For parallel processing you can use a diffent dispatcher (see Extras/BulletMultiThreaded)
	m_dispatcher = new	btCollisionDispatcher(m_collisionConfiguration);

	//m_broadphase = new btDbvtBroadphase();
    btVector3 worldMin(-60,-60,-100);
	btVector3 worldMax(60,60,100);

	btDbvtBroadphase* sweepBP = new btDbvtBroadphase;
	m_broadphase = sweepBP;
	//m_broadphase = new bt32BitAxisSweep3(worldMin,worldMax);

	///the default constraint solver. For parallel processing you can use a different solver (see Extras/BulletMultiThreaded)
	btSequentialImpulseConstraintSolver* sol = new btSequentialImpulseConstraintSolver;
	m_solver = sol;

	//m_dynamicsWorld = new btDiscreteDynamicsWorld(m_dispatcher,m_broadphase,m_solver,m_collisionConfiguration);

	m_dynamicsWorld = new btDiscreteDynamicsWorld(m_dispatcher,m_broadphase,m_solver,m_collisionConfiguration);
	m_dynamicsWorld->getSolverInfo().m_solverMode |=SOLVER_RANDMIZE_ORDER;
	m_dynamicsWorld->getDispatchInfo().m_useContinuous=true;

	m_dynamicsWorld->getDispatchInfo().m_allowedCcdPenetration=0.0001f;
	m_dynamicsWorld->setDebugDrawer(&gDebugDraw);
	m_dynamicsWorld->setGravity(btVector3(0,0,-100));

	m_dynamicsWorld->getSolverInfo().m_splitImpulse = true;
	m_dynamicsWorld->getSolverInfo().m_splitImpulsePenetrationThreshold = 1e30f;
	m_dynamicsWorld->getSolverInfo().m_maxErrorReduction = 1e30f;
	m_dynamicsWorld->getSolverInfo().m_erp  =1.f;
	m_dynamicsWorld->getSolverInfo().m_erp2 = 1.f;




	btCollisionShape* bottomShape = new btBoxShape(btVector3(btScalar(1000.),btScalar(1000.),btScalar(1.)));

	m_collisionShapes.push_back(bottomShape);


    btTransform bottomTransform;
	bottomTransform.setIdentity();
	bottomTransform.setOrigin(btVector3(0,0,-10));

{
		btScalar mass(0.);

		btVector3 localInertia(0,0,0);

		//using motionstate is recommended, it provides interpolation capabilities, and only synchronizes 'active' objects
		btDefaultMotionState* myMotionState = new btDefaultMotionState(bottomTransform);
		btRigidBody::btRigidBodyConstructionInfo rbInfo(mass,myMotionState,bottomShape,localInertia);
        btRigidBody* body = new btRigidBody(rbInfo);
		body->setFriction(1.f);
		body->setRestitution(0.0f);
		body->setUserPointer((void*)&bottomTag);

		//add the body to the dynamics world
		m_dynamicsWorld->addRigidBody(body);


	}
	{
		//create a few dynamic rigidbodies
		// Re-using the same collision is better for memory usage and performance

		btCollisionShape* colShape = new btCylinderShapeZ(btVector3(SCALING*4.f,SCALING*4.f,SCALING*0.7f));
		m_collisionShapes.push_back(colShape);

	}

	pickedBody=0;


}
void	BasicDemo::clientResetScene()
{
	exitPhysics();
	initPhysics();
}


void	BasicDemo::exitPhysics()
{

	//cleanup in the reverse order of creation/initialization

	//remove the rigidbodies from the dynamics world and delete them
	int i;
	for (i=m_dynamicsWorld->getNumCollisionObjects()-1; i>=0 ;i--)
	{
		btCollisionObject* obj = m_dynamicsWorld->getCollisionObjectArray()[i];
		//coin* body = (coin*)btRigidBody::upcast(obj);
		coin* body = dynamic_cast<coin*>(btRigidBody::upcast(obj));
		//btRigidBody* body1 = btRigidBody::upcast(obj);
		if (body!=0)
		{
			for (int j=body->getNumConstraintRefs()-1; j>=0;j--)
			{
				m_dynamicsWorld->removeConstraint(body->getConstraintRef(j));
			}
			if (body && body->getMotionState())
			{
				delete body->getMotionState();
			}
		}


		m_dynamicsWorld->removeCollisionObject( obj );
		delete obj;
	}

	//delete collision shapes
	for (int j=0;j<m_collisionShapes.size();j++)
	{
		btCollisionShape* shape = m_collisionShapes[j];
		delete shape;
	}
	m_collisionShapes.clear();

	delete m_dynamicsWorld;

	delete m_solver;

	delete m_broadphase;

	delete m_dispatcher;

	delete m_collisionConfiguration;


}


bool BasicDemo::isAtRest()

{
    bool atRest = true;
    for(std::map<short,coin*>::iterator c1=coins.begin();c1!=coins.end();c1++)
    {
        if (c1->second->getLinearVelocity().length()>0.05)
        {
	    	btVector3 v1 =c1->second->getLinearVelocity();
            atRest=false;
        }
    };

    return atRest;

}


void BasicDemo::syncCoins()
{
	    std::set<int> ckeys;
		for( std::map<short, coin*>::iterator it = coins.begin(); it != coins.end(); ++it)
        {
			ckeys.insert(it->first);
		}

		std::set<int> luaKeys = luaGetCoinKeys();

		std::set<int> diff;

		std::set_difference(ckeys.begin(),ckeys.end(),luaKeys.begin(), luaKeys.end(),std::inserter(diff,diff.end()));

        //const int	numObjects=m_dynamicsWorld->getNumCollisionObjects();
        for(std::set<int>::iterator i=diff.begin();i!=diff.end();i++)
        {
            coin*	body = coins[*i];

            for (int j=body->getNumConstraintRefs()-1; j>=0;j--)
			{
				m_dynamicsWorld->removeConstraint(body->getConstraintRef(j));
			}
			if (body && body->getMotionState())
			{
				delete body->getMotionState();
			}
			m_dynamicsWorld->removeCollisionObject( body );

            coins.erase(*i);

			delete body;

        }

		for( std::map<short, coin*>::iterator it = coins.begin(); it != coins.end(); it++) {
    		importParamsFromLua(it->first);
		}
}


void BasicDemo::syncZones()
{
	    std::set<int> ckeys;

		for( std::map<short, btPairCachingGhostObject*>::iterator it = zones.begin(); it != zones.end(); ++it)
        {
			ckeys.insert(it->first);
		}

		std::set<int> luaKeys = lua_getKeys("zones");

		std::set<int> diff;

		std::set_difference(ckeys.begin(),ckeys.end(),luaKeys.begin(), luaKeys.end(),std::inserter(diff,diff.end()));

        //const int	numObjects=m_dynamicsWorld->getNumCollisionObjects();
        for(std::set<int>::iterator i=diff.begin();i!=diff.end();i++)
        {
            btPairCachingGhostObject*	zone = zones[*i];
			/*
            for (int j=body->getNumConstraintRefs()-1; j>=0;j--)
			{
				m_dynamicsWorld->removeConstraint(body->getConstraintRef(j));
			}
			if (body && body->getMotionState())
			{
				delete body->getMotionState();
			}
			*/
			m_dynamicsWorld->removeCollisionObject( zone );


            zones.erase(*i);

			delete zone;

        }

}

void BasicDemo::createTerrain()
{

//btRigidBody*	localCreateRigidBody(float mass, const btTransform& startTransform,btCollisionShape* shape);

    btTriangleMesh* groundMesh = new btTriangleMesh(true,false);



    //btVector3 v0 = btVector3(-100.f,-100.f,1.);

    Vector2dVector levelTriangles = lua_triangulate();
    int tcount = levelTriangles.size()/3;



        for (int i=0; i<tcount; i++)
    {
         btVector3 &p1 = levelTriangles[i*3+0];
         btVector3 &p2 = levelTriangles[i*3+1];
         btVector3 &p3 = levelTriangles[i*3+2];
		btVector3 v1 = p2 - p1;
		btVector3 v2 = p3 - p1;

		btVector3 n1=v1.cross(v2);
		if (n1.getZ()<=0.f)
		{
			btVector3& p4 =  p1;

			p1=p3;

			p3=p4;
		}

        //printf("Triangle %d => (%0.0f,%0.0f) (%0.0f,%0.0f) (%0.0f,%0.0f)\n",i+1,p1.getX(),p1.getY(),p2.getX(),p2.getY(),p3.getX(),p3.getY());
        groundMesh->addTriangle(btVector3(p1.getX(),p1.getY(),0.f),btVector3(p2.getX(),p2.getY(),0.f),btVector3(p3.getX(),p3.getY(),0.f),true);
    }


    //groundMesh->addTriangle(btVector3(-50.f,-50.f,0.),btVector3(0.f,50.f,0.),btVector3(50.f,-50.f,0.),true);


    btBvhTriangleMeshShape* groundShape = new btBvhTriangleMeshShape(groundMesh,true);

	btTriangleInfoMap* triangleInfoMap = new btTriangleInfoMap();


	btGenerateInternalEdgeInfo(groundShape,triangleInfoMap);


    m_collisionShapes.push_back(groundShape);

	btTransform groundTransform;
	groundTransform.setIdentity();
	groundTransform.setOrigin(btVector3(0,0,0));

	//We can also use DemoApplication::localCreateRigidBody, but for clarity it is provided here:
	{
		btScalar mass(0.);

		//rigidbody is dynamic if and only if mass is non zero, otherwise static
		bool isDynamic = (mass != 0.f);

		btVector3 localInertia(0,0,0);
		if (isDynamic)
			groundShape->calculateLocalInertia(mass,localInertia);

		//using motionstate is recommended, it provides interpolation capabilities, and only synchronizes 'active' objects
		btDefaultMotionState* myMotionState = new btDefaultMotionState(groundTransform);
		btRigidBody::btRigidBodyConstructionInfo rbInfo(mass,myMotionState,groundShape,localInertia);
		btRigidBody* body = new btRigidBody(rbInfo);
		body->setFriction(1.f);
		body->setRestitution(0.01f);

		//add the body to the dynamics world
		m_dynamicsWorld->addRigidBody(body);
		ground = body;

		ground->setCollisionFlags(ground->getCollisionFlags()  | btCollisionObject::CF_CUSTOM_MATERIAL_CALLBACK);
	}

}


